pub struct TrainingStats {
pub population_stats: PopulationStats,
pub child_count: usize,
}Expand description
A collection of statistics about the population as a whole.
Relatively more expensive to compute than training metrics, so should be computed infrequently.
Fields
population_stats: PopulationStatsA collection of standard population stats: see PopulationStats
for more information
child_count: usizeTotal number of children that have been reproduced and introduced into the population, including the initial seed population count.
Same as TrainingCriteriaMetrics::child_count.