pub struct TrainingReportStrategy<F, C> {
pub should_report: F,
pub report_callback: C,
}Expand description
A struct that encompasses a two-part reporting strategy to use when performing periodic progress updates.
TrainingReportStrategy::should_report
represents whether or not a report should be generated
at this moment, and TrainingReportStrategy::report_callback
performs the actual reporting.
Fields
should_report: FA callback used to determine if a report should be generated at this moment.
report_callback: CA callback used to when a report should be displayed or logged in some form.
This callback is only called if
TrainingReportStrategy::should_report returns true.